/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jcr.taglib;

import com.squeakysand.jcr.SimpleNode;
import com.squeakysand.jsp.tagext.BodyContent;
import com.squeakysand.jsp.tagext.VariableScope;
import com.squeakysand.jsp.tagext.annotations.JspTag;
import com.squeakysand.jsp.tagext.annotations.JspTagAttribute;
import com.squeakysand.jsp.tagext.annotations.JspTagVariable;

import java.io.IOException;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.servlet.jsp.JspException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Iterates over the properties of a JCR node.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
@JspTag(bodyContent = BodyContent.SCRIPTLESS)
public class ForEachPropertyTag extends JcrSimpleTagSupport {

    private static final Logger LOG = LoggerFactory.getLogger(ForEachPropertyTag.class);

    private Node node;
    private String nameVar;
    private String valueVar;

    @Override
    public void doTag() throws JspException, IOException {
        SimpleNode targetNode = determineTargetNode();
        if (targetNode != null) {
            for (Property property : targetNode.getAllProperties()) {
                try {
                    setPageAttribute(nameVar, property.getName());
                    setPageAttribute(valueVar, property.getValue());
                    getJspBody().invoke(null);
                } catch (RepositoryException e) {
                    LOG.error(e.getMessage(), e);
                }
            }
        }
    }

    public String getNameVar() {
        return nameVar;
    }

    public Node getNode() {
        return node;
    }

    public String getValueVar() {
        return valueVar;
    }

    @JspTagVariable(scope = VariableScope.NESTED)
    @JspTagAttribute(required = true)
    public void setNameVar(String nameVar) {
        this.nameVar = nameVar;
    }

    @JspTagAttribute(rtexprvalue = true)
    public void setNode(Node node) {
        this.node = node;
    }

    @JspTagVariable(scope = VariableScope.NESTED, variableClass = Value.class)
    @JspTagAttribute(required = true)
    public void setValueVar(String valueVar) {
        this.valueVar = valueVar;
    }

    private SimpleNode determineTargetNode() {
        SimpleNode result = null;
        Node targetNode = null;
        // first check if a node was passed via an attribute
        if (node != null) {
            LOG.debug("node was passed in via an attribute");
            targetNode = node;
        } else {
            // if no node was passed via an attribute, then see if there is a current node in the pageContext
            try {
                targetNode = (Node) getJspContext().findAttribute(AttributeNames.CURRENT_NODE);
                LOG.debug("node was found via current node attribute");
            } catch (ClassCastException e) {
                LOG.error(e.getMessage(), e);
            }

        }
        LOG.debug("target node was " + targetNode);
        if (targetNode != null) {
            if (targetNode instanceof SimpleNode) {
                result = (SimpleNode) targetNode;
            } else {
                result = new SimpleNode(targetNode);
            }
        }
        LOG.debug("result node was " + result);
        return result;
    }
}
